@@ -21,62 +21,79 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 #include <math.h>
+#include <pthread.h>
 
-#include <X11/Intrinsic.h>
 #ifndef NO_X11
+# include <X11/Xlib.h>
+# include <X11/Intrinsic.h>
 # include <X11/StringDefs.h>
 # include <X11/Xaw/XawInit.h>
 # include <X11/Xaw/Command.h>
 # include <X11/Xaw/Paned.h>
 #endif
 
-#include "grab.h"
+#include "grab-ng.h"
 #include "channel.h"
+#include "commands.h"
 #include "frequencies.h"
 #include "sound.h"
 #include "parseconfig.h"
+#include "event.h"
 
 /* ----------------------------------------------------------------------- */
 /* misc common stuff, not only channel related                             */ 
 
-struct CHANNEL  defaults    = { "defaults", NULL,
-				"5", 0, 0, 0,
-				CAPTURE_OVERLAY, 0, 0,
-				32768, 32768, 32768, 32768 };
+struct CHANNEL defaults = {
+    name:     "defaults",
+    capture:  CAPTURE_ON,
+    channel:  -1,
+    audio:    -1,
+    color:    -1,
+    bright:   -1,
+    hue:      -1,
+    contrast: -1,
+};
+				
 struct CHANNEL  **channels  = NULL;
 int             count       = 0;
 int             alloc_count = 0;
-int             have_mixer  = 0;
 
 int last_sender = -1, cur_sender = -1, cur_channel = -1, cur_fine = 0;
-int cur_norm = -1, cur_input = -1, cur_freq;
+int cur_freq;
+struct ng_filter *cur_filter;
 
-int cur_color,cur_bright,cur_hue,cur_contrast,cur_capture;
-int cur_mute = 0, cur_volume = 65535;
+int cur_capture = CAPTURE_OFF;
 int have_config;
-int jpeg_quality = 75;
-int mjpeg_quality = 50;
 int keypad_ntsc = 0;
+int keypad_partial = 1;
 int use_osd = 1;
 int fs_width,fs_height,fs_xoff,fs_yoff;
 int pix_width=128, pix_height=96, pix_cols=1;
 
-#ifndef NO_X11
-void button_cb(Widget widget, XtPointer clientdata, XtPointer call_data);
+char *mov_driver = NULL;
+char *mov_video  = NULL;
+char *mov_fps    = NULL;
+char *mov_audio  = NULL;
+char *mov_rate   = NULL;
 
+#ifndef NO_X11
 extern Widget chan_box, chan_viewport, tv, opt_paned, launch_paned;
 #endif
 
-char *mixer  = NULL;
+static char *mixer = NULL;
+char mixerdev[32],mixerctl[16];
+char *midi = NULL;
 
 struct LAUNCH *launch = NULL;
-int nlaunch          = 0;
+int nlaunch           = 0;
 
 /* ----------------------------------------------------------------------- */
 
@@ -85,6 +102,9 @@ int lookup_channel(char *channel)
     int    i,nr1,nr2;
     char   tag1[5],tag2[5];
 
+    if (NULL == channel)
+	return -1;
+
     if (isdigit(channel[0])) {
 	tag1[0] = 0;
 	nr1  = atoi(channel);
@@ -112,7 +132,7 @@ int lookup_channel(char *channel)
     return i;
 }
 
-int  get_freq(int i)
+int get_freq(int i)
 {
     if (i < 0 || i >= chancount)
 	return -1;
@@ -188,7 +208,8 @@ void hotkey_channel(struct CHANNEL *channel)
     XtOverrideTranslations(chan_viewport,XtParseTranslationTable(str));
 }
 
-void launch_cb(Widget widget, XtPointer clientdata, XtPointer call_data)
+static void
+launch_cb(Widget widget, XtPointer clientdata, XtPointer call_data)
 {
     char *argv[2];
 
@@ -197,7 +218,8 @@ void launch_cb(Widget widget, XtPointer clientdata, XtPointer call_data)
     XtCallActionProc(widget,"Launch",NULL,argv,1);
 }
 
-void hotkey_launch(struct LAUNCH *launch)
+static void
+hotkey_launch(struct LAUNCH *launch)
 {
     Widget c;
     char str[100],key[32],ctrl[16],label[64];
@@ -222,6 +244,13 @@ void hotkey_launch(struct LAUNCH *launch)
     XtAddCallback(c,XtNcallback,launch_cb,(XtPointer)(launch->name));
 }
 
+static void
+button_cb(Widget widget, XtPointer clientdata, XtPointer call_data)
+{
+    struct CHANNEL *channel = clientdata;
+    do_va_cmd(2,"setstation",channel->name);
+}
+
 /* ... and initalize later */
 void configure_channel(struct CHANNEL *channel)
 {
@@ -240,9 +269,6 @@ void configure_channel(struct CHANNEL *channel)
 void
 del_channel(int i)
 {
-#ifndef NO_X11
-    XtDestroyWidget(channels[i]->button);
-#endif
     free(channels[i]->name);
     if (channels[i]->key)
 	free(channels[i]->key);
@@ -258,6 +284,8 @@ calc_frequencies()
     int i;
 
     for (i = 0; i < count; i++) {
+	if (NULL == channels[i]->cname)
+	    continue;
 	channels[i]->channel = lookup_channel(channels[i]->cname);
 	if (-1 == channels[i]->channel)
 	    channels[i]->freq = -1;
@@ -272,6 +300,7 @@ calc_frequencies()
 static void
 init_channel(char *name, struct CHANNEL *c)
 {
+    struct ng_attribute *attr;
     char *val; int n,i;
 
     if (NULL != (val = cfg_get_str(name,"capture"))) {
@@ -280,65 +309,92 @@ init_channel(char *name, struct CHANNEL *c)
 	else
 	    fprintf(stderr,"config: invalid value for capture: %s\n",val);
     }
-    if (NULL != (val = cfg_get_str(name,"input")) ||
-	NULL != (val = cfg_get_str(name,"source"))) { /* obsolete */
-	if (-1 != (i = str_to_int(val,grabber->inputs)))
+    if (NULL != (attr = ng_attr_byid(attrs,ATTR_ID_INPUT)) &&
+	(NULL != (val = cfg_get_str(name,"input")) ||
+	 NULL != (val = cfg_get_str(name,"source")))) { /* obsolete */
+	if (-1 != (i = ng_attr_getint(attr,val)))
 	    c->input = i;
-	else
+	else {
 	    fprintf(stderr,"config: invalid value for input: %s\n",val);
+	    ng_attr_listchoices(attr);
+	}
     }
-    if (NULL != (val = cfg_get_str(name,"norm"))) {
-	if (-1 != (i = str_to_int(val,grabber->norms)))
+    if (NULL != (attr = ng_attr_byid(attrs,ATTR_ID_NORM)) &&
+	NULL != (val = cfg_get_str(name,"norm"))) {
+	if (-1 != (i = ng_attr_getint(attr,val)))
 	    c->norm = i;
-	else
+	else {
 	    fprintf(stderr,"config: invalid value for norm: %s\n",val);
+	    ng_attr_listchoices(attr);
+	}
+    }
+    if (NULL != (attr = ng_attr_byid(attrs,ATTR_ID_AUDIO_MODE)) &&
+	NULL != (val = cfg_get_str(name,"audio"))) {
+	if (-1 != (i = ng_attr_getint(attr,val)))
+	    c->audio = i;
+	else {
+	    fprintf(stderr,"config: invalid value for audio: %s\n",val);
+	    ng_attr_listchoices(attr);
+	}
     }
     
     if (NULL != (val = cfg_get_str(name,"channel")))
 	c->cname   = strdup(val);
-    if (-1 != (n = cfg_get_int(name,"fine")))
+    if (NULL != (val = cfg_get_str(name,"freq")))
+	c->freq   = (int)(atof(val)*16);
+    if (0 != (n = cfg_get_signed_int(name,"fine")))
 	c->fine = n;
 
     if (NULL != (val = cfg_get_str(name,"key")))
-	c->key   = strdup(val);
+	c->key  = strdup(val);
+    if (NULL != (val = cfg_get_str(name,"midi")))
+	c->midi = atoi(val);
 
-    if (-1 != (n = cfg_get_int(name,"color")))
-	c->color = n;
-    if (-1 != (n = cfg_get_int(name,"bright")))
-	c->bright = n;
-    if (-1 != (n = cfg_get_int(name,"hue")))
-	c->hue = n;
-    if (-1 != (n = cfg_get_int(name,"contrast")))
-	c->contrast = n;
+    attr = ng_attr_byid(attrs,ATTR_ID_COLOR);
+    if (attr && NULL != (val = cfg_get_str(name,"color")))
+	c->color = ng_attr_parse_int(attr,val);
+    attr = ng_attr_byid(attrs,ATTR_ID_BRIGHT);
+    if (attr && NULL != (val = cfg_get_str(name,"bright")))
+	c->bright = ng_attr_parse_int(attr,val);
+    attr = ng_attr_byid(attrs,ATTR_ID_HUE);
+    if (attr && NULL != (val = cfg_get_str(name,"hue")))
+	c->hue = ng_attr_parse_int(attr,val);
+    attr = ng_attr_byid(attrs,ATTR_ID_CONTRAST);
+    if (attr && NULL != (val = cfg_get_str(name,"contrast")))
+	c->contrast = ng_attr_parse_int(attr,val);
 }
 
 void
-read_config()
+read_config(char *conffile)
 {
-    char filename[100], key[16], cmdline[128];
-    char mixerdev[32],mixerctl[16];
-    char **list,*val;
+    char filename[100];
+    char *val;
     int  i;
 
-    cfg_parse_file("/usr/X11R6/lib/X11/xawtvrc");
-    sprintf(filename,"%s/%s",getenv("HOME"),".xawtv");
-    if (0 == cfg_parse_file(filename))
-	have_config = 1;
+    if (conffile) {
+	if (0 == cfg_parse_file(conffile))
+	    have_config = 1;
+    } else {
+	sprintf(filename,"%s/%s",getenv("HOME"),".xawtv");
+	if (0 == cfg_parse_file(CONFIGFILE))
+	    have_config = 1;
+	if (0 == cfg_parse_file(filename))
+	    have_config = 1;
+    }
 
     /* misc global settings */
     if (NULL != (val = cfg_get_str("global","mixer"))) {
 	mixer = strdup(val);
 	if (2 != sscanf(mixer,"%31[^:]:%15s",mixerdev,mixerctl)) {
-	    strcpy(mixerdev,"/dev/mixer");
+	    strcpy(mixerdev,ng_dev.mixer);
 	    strncpy(mixerctl,val,15);
 	    mixerctl[15] = 0;
 	}
-	if (-1 != mixer_open(mixerdev, mixerctl))
-	    have_mixer = 1;
-	else
-	    fprintf(stderr,"invalid value for mixer: %s\n",val);
     }
 
+    if (NULL != (val = cfg_get_str("global","midi")))
+	midi = strdup(val);
+
     if (NULL != (val = cfg_get_str("global","freqtab"))) {
 	for (i = 0; chanlists[i].name != NULL; i++)
 	    if (0 == strcasecmp(val,chanlists[i].name))
@@ -374,21 +430,49 @@ read_config()
 	    fs_xoff = fs_yoff = 0;
 	}
     }
-	
+    if (NULL != (val = cfg_get_str("global","ratio"))) {
+	if (2 != sscanf(val,"%d:%d",&ng_ratio_x,&ng_ratio_y)) {
+	    fprintf(stderr,"invalid value for ratio: %s\n",val);
+	    ng_ratio_x = ng_ratio_y = 0;
+	}
+    }
+    
     if (-1 != (i = cfg_get_int("global","jpeg-quality")))
-	jpeg_quality = i;
-    if (-1 != (i = cfg_get_int("global","mjpeg-quality")))
-	mjpeg_quality = i;
+	ng_jpeg_quality = i;
 
     if (NULL != (val = cfg_get_str("global","keypad-ntsc")))
 	if (-1 != (i = str_to_int(val,booltab)))
 	    keypad_ntsc = i;
+    if (NULL != (val = cfg_get_str("global","keypad-partial")))
+	if (-1 != (i = str_to_int(val,booltab)))
+	    keypad_partial = i;
     if (NULL != (val = cfg_get_str("global","osd")))
 	if (-1 != (i = str_to_int(val,booltab)))
 	    use_osd = i;
 
+    if (NULL != (val = cfg_get_str("global","mov-driver")))
+	mov_driver = val;
+    if (NULL != (val = cfg_get_str("global","mov-video")))
+	mov_video = val;
+    if (NULL != (val = cfg_get_str("global","mov-fps")))
+	mov_fps = val;
+    if (NULL != (val = cfg_get_str("global","mov-audio")))
+	mov_audio = val;
+    if (NULL != (val = cfg_get_str("global","mov-rate")))
+	mov_rate = val;
+}
+
+void
+parse_config(void)
+{
+    char key[16], cmdline[128];
+    char **list,*val;
+#ifndef NO_X11
+    int i;
+#endif
+
     /* launch */
-    list = list = cfg_list_entries("launch");
+    list = cfg_list_entries("launch");
     if (NULL != list) {
 	for (; *list != NULL; list++) {
 	    if (NULL != (val = cfg_get_str("launch",*list)) &&
@@ -408,12 +492,16 @@ read_config()
 	}
     }
 
+    /* events */
+    event_readconfig();
+
     /* channels */
     init_channel("defaults",&defaults);
     for (list = cfg_list_sections(); *list != NULL; list++) {
 	if (0 == strcmp(*list,"defaults")) continue;
 	if (0 == strcmp(*list,"global"))   continue;
 	if (0 == strcmp(*list,"launch"))   continue;
+	if (0 == strcmp(*list,"eventmap")) continue;
 	init_channel(*list,add_channel(*list));
     }
 
@@ -432,6 +520,7 @@ read_config()
 void
 save_config()
 {
+    struct ng_attribute *attr;
     char filename1[100], filename2[100];
     FILE *fp;
     int i;
@@ -458,15 +547,31 @@ save_config()
 	fprintf(fp,"fullscreen = %d x %d\n",fs_width,fs_height);
     if (fs_xoff || fs_yoff)
 	fprintf(fp,"wm-off-by = %+d%+d\n",fs_xoff,fs_yoff);
+    if (ng_ratio_x || ng_ratio_y)
+	fprintf(fp,"ratio = %d:%d\n",ng_ratio_x,ng_ratio_y);
     fprintf(fp,"freqtab = %s\n",chanlists[chantab].name);
     fprintf(fp,"pixsize = %d x %d\n",pix_width,pix_height);
     fprintf(fp,"pixcols = %d\n",pix_cols);
-    fprintf(fp,"jpeg-quality = %d\n",jpeg_quality);
-    fprintf(fp,"mjpeg-quality = %d\n",mjpeg_quality);
+    fprintf(fp,"jpeg-quality = %d\n",ng_jpeg_quality);
     fprintf(fp,"keypad-ntsc = %s\n",int_to_str(keypad_ntsc,booltab));
+    fprintf(fp,"keypad-partial = %s\n",int_to_str(keypad_partial,booltab));
     fprintf(fp,"osd = %s\n",int_to_str(use_osd,booltab));
     if (mixer)
 	fprintf(fp,"mixer = %s\n",mixer);
+    if (midi)
+	fprintf(fp,"midi = %s\n",midi);
+
+    if (mov_driver)
+	fprintf(fp,"mov-driver = %s\n",mov_driver);
+    if (mov_video)
+	fprintf(fp,"mov-video = %s\n",mov_video);
+    if (mov_fps)
+	fprintf(fp,"mov-fps = %s\n",mov_fps);
+    if (mov_audio)
+	fprintf(fp,"mov-audio = %s\n",mov_audio);
+    if (mov_rate)
+	fprintf(fp,"mov-rate = %s\n",mov_rate);
+    
     fprintf(fp,"\n");
     
     if (nlaunch > 0) {
@@ -478,51 +583,94 @@ save_config()
 	fprintf(fp,"\n");
     }
 
+    /* events */
+    event_writeconfig(fp);
+
+    /* write help */
+    fprintf(fp,"# [Station name]\n");
+    fprintf(fp,"# capture = overlay | grabdisplay | on | off\n");
+    fprintf(fp,"# input = Television | Composite1 | S-Video | ...\n");
+    fprintf(fp,"# norm = PAL | NTSC | SECAM | ... \n");
+    fprintf(fp,"# channel = #\n");
+    fprintf(fp,"# fine = # (-128..+127)\n");
+    fprintf(fp,"# key = keysym | modifier+keysym\n");
+    fprintf(fp,"# color = #\n");
+    fprintf(fp,"# bright = #\n");
+    fprintf(fp,"# hue = #\n");
+    fprintf(fp,"# contrast = #\n");
+    fprintf(fp,"\n");
+
     /* write defaults */
     fprintf(fp,"[defaults]\n");
-    fprintf(fp,"norm = %s\n",int_to_str(cur_norm,grabber->norms));
+    fprintf(fp,"norm = %s\n",
+	    ng_attr_getstr(ng_attr_byid(attrs,ATTR_ID_NORM),
+			   cur_attrs[ATTR_ID_NORM]));
+    fprintf(fp,"input = %s\n",
+	    ng_attr_getstr(ng_attr_byid(attrs,ATTR_ID_INPUT),
+			   cur_attrs[ATTR_ID_INPUT]));
     fprintf(fp,"capture = %s\n",int_to_str(cur_capture,captab));
-    fprintf(fp,"source = %s\n",
-	    int_to_str(cur_input,grabber->inputs));
-    if (cur_color != 32768)
-	fprintf(fp,"color = %d\n",cur_color);
-    if (cur_bright != 32768)
-	fprintf(fp,"bright = %d\n",cur_bright);
-    if (cur_hue != 32768)
-	fprintf(fp,"hue = %d\n",cur_hue);
-    if (cur_contrast != 32768)
-	fprintf(fp,"contrast = %d\n",cur_contrast);
+
+    attr = ng_attr_byid(attrs,ATTR_ID_COLOR);
+    if (attr && attr->defval != cur_attrs[ATTR_ID_COLOR])
+	fprintf(fp,"color = %d%%\n",
+		ng_attr_int2percent(attr,cur_attrs[ATTR_ID_COLOR]));
+    attr = ng_attr_byid(attrs,ATTR_ID_BRIGHT);
+    if (attr && attr->defval != cur_attrs[ATTR_ID_BRIGHT])
+	fprintf(fp,"bright = %d%%\n",
+		ng_attr_int2percent(attr,cur_attrs[ATTR_ID_BRIGHT]));
+    attr = ng_attr_byid(attrs,ATTR_ID_HUE);
+    if (attr && attr->defval != cur_attrs[ATTR_ID_HUE])
+	fprintf(fp,"hue = %d%%\n",
+		ng_attr_int2percent(attr,cur_attrs[ATTR_ID_HUE]));
+    attr = ng_attr_byid(attrs,ATTR_ID_CONTRAST);
+    if (attr && attr->defval != cur_attrs[ATTR_ID_CONTRAST])
+	fprintf(fp,"contrast = %d%%\n",
+		ng_attr_int2percent(attr,cur_attrs[ATTR_ID_CONTRAST]));
     fprintf(fp,"\n");
 
     /* write channels */
     for (i = 0; i < count; i++) {
-
 	fprintf(fp,"[%s]\n",channels[i]->name);
-	fprintf(fp,"channel = %s\n",chanlist[channels[i]->channel].name);
-	if (0 != channels[i]->fine)
-	    fprintf(fp,"fine = %+d\n", channels[i]->fine);
-	if (cur_norm != channels[i]->norm)
+	if (0 != strcmp(channels[i]->cname,"none")) {
+	    fprintf(fp,"channel = %s\n",chanlist[channels[i]->channel].name);
+	    if (0 != channels[i]->fine)
+		fprintf(fp,"fine = %+d\n", channels[i]->fine);
+	} else {
+	    fprintf(fp,"freq = %.2f\n",(float)(channels[i]->freq)/16);
+	}
+	
+	if ( channels[i]->norm != cur_attrs[ATTR_ID_NORM])
 	    fprintf(fp,"norm = %s\n",
-		    int_to_str(cur_norm,grabber->norms));
+		    ng_attr_getstr(ng_attr_byid(attrs,ATTR_ID_NORM),
+				   channels[i]->norm));
+	if (channels[i]->input != cur_attrs[ATTR_ID_INPUT])
+	    fprintf(fp,"input = %s\n",
+		    ng_attr_getstr(ng_attr_byid(attrs,ATTR_ID_INPUT),
+				   channels[i]->input));
 	if (channels[i]->key != NULL)
 	    fprintf(fp,"key = %s\n",channels[i]->key);
-#if 0
+	if (channels[i]->midi != 0)
+	    fprintf(fp,"midi = %d\n",channels[i]->midi);
 	if (channels[i]->capture != cur_capture)
 	    fprintf(fp,"capture = %s\n",
 		    int_to_str(channels[i]->capture,captab));
-#endif
-	if (channels[i]->input != cur_input)
-	    fprintf(fp,"input = %s\n",
-		    int_to_str(channels[i]->input,grabber->inputs));
-	  
-	if (cur_color != channels[i]->color)
-	    fprintf(fp,"color = %d\n",channels[i]->color);
-	if (cur_bright != channels[i]->bright)
-	    fprintf(fp,"bright = %d\n",channels[i]->bright);
-	if (cur_hue != channels[i]->hue)
-	    fprintf(fp,"hue = %d\n",channels[i]->hue);
-	if (cur_contrast != channels[i]->contrast)
-	    fprintf(fp,"contrast = %d\n",channels[i]->contrast);
+	
+	attr = ng_attr_byid(attrs,ATTR_ID_COLOR);
+	if (attr && cur_attrs[ATTR_ID_COLOR] != channels[i]->color)
+	    fprintf(fp,"color = %d%%\n",
+		    ng_attr_int2percent(attr,channels[i]->color));
+	attr = ng_attr_byid(attrs,ATTR_ID_BRIGHT);
+	if (attr && cur_attrs[ATTR_ID_BRIGHT] != channels[i]->bright)
+	    fprintf(fp,"bright = %d%%\n",
+		    ng_attr_int2percent(attr,channels[i]->bright));
+	attr = ng_attr_byid(attrs,ATTR_ID_HUE);
+	if (attr && cur_attrs[ATTR_ID_HUE] != channels[i]->hue)
+	    fprintf(fp,"hue = %d%%\n",
+		    ng_attr_int2percent(attr,channels[i]->hue));
+	attr = ng_attr_byid(attrs,ATTR_ID_CONTRAST);
+	if (attr && cur_attrs[ATTR_ID_CONTRAST] != channels[i]->contrast)
+	    fprintf(fp,"contrast = %d%%\n",
+		    ng_attr_int2percent(attr,channels[i]->contrast));
 
 	fprintf(fp,"\n");
     }
@@ -554,7 +702,7 @@ str_to_int(char *str, struct STRTAB *tab)
     return -1;
 }
 
-char*
+const char*
 int_to_str(int n, struct STRTAB *tab)
 {
     int i;
@@ -564,4 +712,3 @@ int_to_str(int n, struct STRTAB *tab)
 	    return tab[i].str;
     return NULL;
 }
-
